/*
 *  $Id: ctc.cc,v 1.1.1.1 1998/01/11 00:00:37 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of KCemu, the KC 85/4 Emulator.
 */

#include <iostream.h>
#include <iomanip.h>

#include "kc/ctc.h"

static int key_table[] = {
    /* 00 */ 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, /*          */
    /* 08 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x7f, 0x00, 0x00, /*          */
    /* 10 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* 18 */ 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0x00, 0x00, /*          */

    /* 20 */ 0x47, 0x74, 0x04, 0x14, 0x64, 0x24, 0x54, 0x34, /*  !"#$%&' "*/
    /* 28 */ 0x44, 0x3a, 0x1a, 0x69, 0x4b, 0x0b, 0x5b, 0x6b, /* ()*+,-./ */ 
    /* 30 */ 0x2b, 0x75, 0x05, 0x15, 0x65, 0x25, 0x55, 0x35, /* 01234567 */
    /* 38 */ 0x45, 0x3b, 0x1b, 0x68, 0x4a, 0x0a, 0x5a, 0x6a, /* 89:;<=>? */
    /* 40 */ 0x2a, 0x02, 0x5e, 0x6e, 0x62, 0x10, 0x22, 0x52, /* @ABCDEFG */
    /* 48 */ 0x32, 0x40, 0x42, 0x48, 0x58, 0x4e, 0x3e, 0x36, /* HIJKLMNO */
    /* 50 */ 0x26, 0x70, 0x60, 0x12, 0x20, 0x30, 0x2e, 0x00, /* PQRSTUVW */
    /* 58 */ 0x1e, 0x0e, 0x50,   -1, 0x67,   -1, 0x17, 0x28, /* XYZ[\]^_ */
    /* 60 */   -1, 0x03, 0x5f, 0x6f, 0x63, 0x11, 0x23, 0x53, /* `abcdefg */
    /* 68 */ 0x33, 0x41, 0x43, 0x49, 0x59, 0x4f, 0x3f, 0x37, /* hijklmno */
    /* 70 */ 0x27, 0x71, 0x61, 0x13, 0x21, 0x31, 0x2f, 0x01, /* pqrstuvw */
    /* 78 */ 0x1f, 0x0f, 0x51,   -1, 0x67,   -1, 0x17, 0x28, /* xyz{|}~<DEL> */

    /* 80 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* 88 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* 90 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* 98 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* a0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* a8 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* b0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* b8 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* c0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* c8 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* d0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* d8 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* e0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* e8 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* f0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */
    /* f8 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /*          */

	      /* <CUL><CUR><CUU><CUD><HOME><CLS><CLR>         */
    /* 100 */ 0x07, 0x7b, 0x79, 0x77, 0x09, 0x08, 0x19, 0x00,
};

CTC::CTC(void)
{
    int a;

    _state   = 0;
    _irq_vector = 0;
    _first_key = _last_key = 0;
    for (a = 0;a < 4;a++) {
	_control[a]     = 0x23;
	_value[a]       = 0;
	_restart[a]     = 0;
	_timer_value[a] = 0;
    }
}

void CTC::keyPressed(int key)
{
    //cout << "keyPressed (" << hex << (int)key << ") [" << (char)key << "]\n";

    key = key_table[key];
    if (key == -1) {
	//cout << "unknown key!\n";
	return;
    }

    if (((_last_key + 1) % MAX_KEYS) == _first_key) {
	//cout << "buffer full\n";
	return;
    }

    _keys[_last_key] = key;
    _last_key = (_last_key + 1) % MAX_KEYS;
}

byte_t CTC::checkInt(void)
{
    int a;
    byte_t key;
    static int pending_pio_irq = 0;

    if (pending_pio_irq) {
	pending_pio_irq = 0;
	//cout << "CTC: fake PIO irq\n";
	return 0xe6; /* fake pio irq */
    }

    if (_state == 0) {
	if (_last_key != _first_key) {
	    key = _keys[_first_key];
	    _first_key = (_first_key + 1) % MAX_KEYS;
	    for (a = 1;a < 8;a++) {
		_buf[a] = (key & 1) ? 0x60 : 0x6e;
		key >>= 1;
	    }
	    _buf[ 0] = 0x8f;
	    _buf[ 8] = 0x3c;
	    _state = MAX_STATE;
	}
    } else {
	pending_pio_irq = 1;
    }

    for (a = 0;a < 4;a++) {
	if (_control[a] & DO_RESET) continue;
	if (_restart[a]) {
	    _value[a] = _timer_value[a];
	    _restart[a] = 0;
	}
	_value[a]--;
	//cout << "CTC: cannel " << (int)a << ": "
	//<< hex << (int)_value[a] << "\n";
	if (_value[a] == 0) {
	    _restart[a] = 1;
	    if (_control[a] & IRQ_ENABLED) {
		//cout << "CTC: irq channel " << (int)a << "\n";
		return _irq_vector | (a << 1);
	    }
	}
    }

    return 0xff;
}

byte_t CTC::in(byte_t channel)
{
    int val;

    if (channel != 3) {
	return _value[channel];
    }

    if (_state > 0) {
	_state--;
	val = _buf[MAX_STATE - _state];
    } else {
	val = 0;
    }
    //cout << "CTC::in(): " << hex << val << "\n";

    return val;
}

void CTC::out(byte_t channel, byte_t val)
{
    if (_control[channel] & WAIT_FOR_TV) {
	_control[channel] &= ~(WAIT_FOR_TV | DO_RESET);
	_timer_value[channel] = val;
	_value[channel] = val;
	//cout << "CTC: new timer value for channel " << (int)channel << ": 0x"
	//<< hex << (int)val << "\n";
	return;
    }

    if ((val & 1) == 0) {
	if (channel != 0) return;
	//cout << "CTC: new irq vector: 0x" << hex << (int)val << "\n";
	_irq_vector = val & ~ 7;
	return;
    }

    //cout << "CTC: new control byte for channel " << (int)channel << ": 0x"
    //<< hex << (int)val << "\n";
    _control[channel] = val;
}
