/*
 *  $Id: ui_dos.cc,v 1.1.1.1 1998/01/11 00:00:37 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of KCemu, the KC 85/4 Emulator.
 */

#include <iostream.h>

#include <allegro.h>

#include "kc/kc.h"
#include "kc/z80.h"
#include "kc/ctc.h"

#include "ui/ui_dos.h"

UI_DOS::UI_DOS(int *argc, char ***argv)
{
	int a;
	RGB colors[] = {
		{ 0x00, 0x00, 0x00 },
		{ 0x00, 0x00, 0xd0 },
		{ 0xd0, 0x00, 0x00 },
		{ 0xd0, 0x00, 0xd0 },
		{ 0x00, 0xd0, 0x00 },
		{ 0x00, 0xd0, 0xd0 },
		{ 0xd0, 0x00, 0x00 },
		{ 0xd0, 0xd0, 0xd0 },

		{ 0x00, 0x00, 0x00 },
		{ 0x60, 0x00, 0xa0 },
		{ 0xa0, 0x60, 0x00 },
		{ 0xa0, 0x00, 0x60 },
		{ 0x00, 0xa0, 0x60 },
		{ 0x00, 0x60, 0xa0 },
		{ 0x30, 0xa0, 0x30 },
		{ 0xd0, 0xd0, 0xd0 },

		{ 0x00, 0x00, 0x00 },
		{ 0x00, 0x00, 0xa0 },
		{ 0xa0, 0x00, 0x00 },
		{ 0xa0, 0x00, 0xa0 },
		{ 0x00, 0xa0, 0x00 },
		{ 0x00, 0xa0, 0xa0 },
		{ 0xa0, 0xa0, 0x00 },
		{ 0xa0, 0xa0, 0xa0 }
	};

	_pix_mem = new char[256 * 40];
	_col_mem = new char[256 * 40];

	allegro_init();
	install_keyboard();
	if (set_gfx_mode(GFX_AUTODETECT, 360, 270, 0, 0) == -1) {
		if (set_gfx_mode(GFX_AUTODETECT, 376, 282, 0, 0) == -1) {
			if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) == -1) {
				cerr << "couldn't set graphic mode!" << endl;
				exit(1);
			}
		}
	}

	_x_offset = (SCREEN_W - 320) / 2;
	_y_offset = (SCREEN_H - 256) / 2;

        for (a = 0;a < 24;a++) {
		colors[a].r /= 4;
		colors[a].g /= 4;
		colors[a].b /= 4;
		set_color(a, &colors[a]);
                _col[a] = a;
        }
}

UI_DOS::~UI_DOS(void)
{
	set_pallete(desktop_pallete);
	allegro_exit();
}

void
UI_DOS::update(bool force_update)
{
}

void
UI_DOS::putPixel(int x, int y, char val)
{
	int a, c;
	long fg, bg;

	//printf("putPixel: %d, %d -> %d\n", x, y, val);

	a = (x << 5) | y;
	_pix_mem[a] = val;
	c = _col_mem[a];
	bg = _col[(c & 7) | 0x10];
	fg = _col[(c >> 3) & 15];

	x += _x_offset;
	y += _y_offset;
	for (a = 0;a < 8;a++) {
		putpixel(screen, x + a, y, (val & 0x80) ? fg : bg);
		val <<= 1;
	}

	_dirty = true;
}

void
UI_DOS::putColor(int x, int y, char val)
{
	int a, c;
	long fg, bg;

	//printf("putColor: %d, %d -> %d\n", x, y, val);

	a = (x << 5) | y;
	_col_mem[a] = val;
	bg = _col[(val & 7) | 0x10];
	fg = _col[(val >> 3) & 15];
	c = _pix_mem[a];

	x += _x_offset;
	y += _y_offset;
	for (a = 0;a < 8;a++) {
		putpixel(screen, x + a, y, (c & 0x80) ? fg : bg);
		c <<= 1;
	}

	_dirty = true;
}

void
UI_DOS::processEvents(void)
{
	int key;

        if (keypressed()) {
		key = readkey();
		switch (key >> 8) {
		case KEY_LEFT:		key = 0x100; break;
		case KEY_RIGHT:		key = 0x101; break;
		case KEY_UP:		key = 0x102; break;
		case KEY_DOWN:		key = 0x103; break;
		case KEY_ESC:		key = 0x01b; break;
		case KEY_BACKSPACE:	key = 0x106; break;
		case KEY_F12:		key = 0x003; break;
		case KEY_HOME:		if (key_shifts & KB_SHIFT_FLAG) {
						key = 0x105;
 					} else {
 						key = 0x104;
 					}
 					break;
		case KEY_DEL:		if (key_shifts & KB_SHIFT_FLAG) {
						key = 0x02;
					} else {
						key = 0x1f;
					}
					break;
		case KEY_F1:		if (key_shifts & KB_SHIFT_FLAG) {
						key = 0xf7;
					} else {
						key = 0xf1;
					}
					break;
		case KEY_F2:		if (key_shifts & KB_SHIFT_FLAG) {
						key = 0xf8;
					} else {
						key = 0xf2;
					}
					break;
		case KEY_F3:		if (key_shifts & KB_SHIFT_FLAG) {
						key = 0xf9;
					} else {
						key = 0xf3;
					}
					break;
		case KEY_F4:		if (key_shifts & KB_SHIFT_FLAG) {
						key = 0xfa;
					} else {
						key = 0xf4;
					}
					break;
		case KEY_F5:		if (key_shifts & KB_SHIFT_FLAG) {
						key = 0xfb;
					} else {
						key = 0xf5;
					}
					break;
		case KEY_F6:		if (key_shifts & KB_SHIFT_FLAG) {
						key = 0xfc;
					} else {
						key = 0xf6;
					}
					break;
		case KEY_INSERT:	if (key_shifts & KB_SHIFT_FLAG) {
						key = 0x14;
					} else {
						key = 0x1a;
					}
					break;
		default:		key &= 0xff;
			                break;

		case KEY_F9:		//theProcessor.setTraceLevel(1);
					clear_keybuf();
					return;
		case KEY_F10:		//fprintf(stderr, "*** RESET ***\n");
					z80->reset();
					clear_keybuf();
					return;
		case KEY_F11:		//fprintf(stderr, "*** POWER ON ***\n");
					z80->powerOn();
					clear_keybuf();
					return;
		case KEY_PAUSE:
		case KEY_PRTSCR:	/*
					 *  EXIT
					 */
					z80->quit();
					return;
		}
		clear_keybuf();
		ctc->keyPressed(key);
	}
}

